/* a smart file splitter program 
5/28/87 George Marshall

ssplit

Purpose: split a file into a set of smaller files.  The file is split
each time the specified pattern (a regular expression, explained below)
is encountered, so each output file (except the first) will therefore 
begin with the specified pattern.  Differs from programs like "chop"
that split a file based on character or line count only.

Usage: 
	ssplit fname reg-ex
		
Where:	fname is the input file name, and 
	reg-ex is the regular expression.

	The output file names have the same first part as the input,
	but have sequence numbers for the extension (two digits).
	(i.e., if the input is ssplit.c, the output files will be
	ssplit.01, ssplit.02, etc.)
	Warning! Don't use an input file with the same name as one of
	the potential output files!

		Note: the simplest regular expression is a simple string.

		Some help on regular expressions (slight liberties taken
		from Unix ... reg ex routines adapted from other sources):
			
		^  = beginning of line
		$  = end of line
		.  = any character except newline
		:a = any alphabetic character
		:d = any digit
		:n = any alphanumeric
		:_ = any white space character
		*  = zero or more occurrences of previous character
		+  = one or more occurrences
		-  = previous character optional
		\  = escape (match following metacharacter literally)
		[]   enclose character set (form: [abcde] or [a-e])

	Examples:
	(1)	ssplit messages ^from		will split up your
		messages file into smaller files, starting a new file
		each time the word "from" is found at the beginning 
		of the line it appears in.
	(2)	ssplit mydoc.txt fro[d-g]	will break up the
		file "mydoc.txt" into files that start with "frod",
		"froe", "frof", or "frog" anywhere in the line.
*/

#include <stdio.h>
char	version[] = {"@(#) ssplit Version 1.0 5-28-87"};

#include "regex.c"

FILE *istream, *ostream;
char	outname[100] = {0};		/* name of current output file */
int	file_is_open = 0;		/* flag: is an output file open? */
int	filenum = 0;			/* suffix value for current outfile */
char	*inname = 0;
#ifdef DEBUG
char deb_pat[] = {"DEBUG is on\n"};
#endif

#define	NEWLINE	0x0a
#define FALSE 0
#define TRUE 1

#define LINELEN	512
char	lbuf[LINELEN]={'A'};		/* the input buffer: spec'd by regex*/

int	lineno = 0;

next_file()
{
	if (file_is_open) {
		if (EOF == fclose(ostream)) {
			printf("Error on close\n");
			exit(-1);
		}
	}
	filenum++;				/* bump file suffix, and 
						generate the next file name */
	sprintf(outname, "%s.%02d", inname, filenum);

	file_is_open = TRUE;			/* turn on flag so this file 
						will get closed next time. */

	if ((ostream = fopen (outname, "w")) == NULL) {

		printf ("Cannot open output file %s", outname);

		exit(-1);
	}
}
#ifdef MSDOS
char	pattern[]={"MSDOS is on\n"};

char	comp_str[512] = {0};
#define re_comp compile
#define re_exec match
#endif MSDOS
char	*temptr = 0;

main (argc, argv)
int argc;
char *argv[];
{
	int	i, maxchar;

	if (argc <= 2) {
		printf("Usage: ssplit <input_file> <reg_ex>");
		return;
	}
	if ((istream = freopen (argv[1], "r", stdin)) == NULL) {
		printf ("Cannot open input file %s", argv[1]);
		return;
	}
	inname = argv[1];		/* get file name for use by get_next*/
					/* find '.' in file name, if any,
					and put in terminator. */
	for (temptr = inname ; *temptr != '\0' ; temptr++) {
		if (*temptr == '.') *temptr = '\0';
	}

	next_file();			/* open first output file */

	re_comp(argv[2]);		/* next arg is reg ex */

	while ((fgets (lbuf, LINELEN, istream)) != (char *)NULL) {
		lineno++;
		maxchar = strlen(lbuf) - 1;
					/* check each line to see if there is 
					a match on the regular expression 
					supplied.  If so, close out the old 
					file, and open the next one for 
					writing. SPECIAL CASE: don't open a 
					new file if we have a match on the 
					first line of the input file, as that 
					would cause a null-length first file*/
		if (re_exec(lbuf) && (lineno != 1) ) {
			next_file();	
		}
		fprintf(ostream, "%s", lbuf); /* write line to file */
	}
}
